<?php

namespace Controller;

use Service\Traits\TResponse;
use Zeedhi\Framework\DataSource\Manager;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Util\Utilities;

class GoalNotifications extends Crud {

    use TResponse;

    protected $dataSourceName = 'goalNotifications';

    /** @var \Service\Goals $goalsService */
    protected $goalsService;
    /** @var \Service\widget $viewService */
    protected $viewService;
    /** @var \Service\widget $goalNotificationsService */
    protected $goalNotificationsService;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService,
                                $goalsService, $goalNotificationsService, $viewService) {
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->goalsService = $goalsService;
        $this->viewService = $viewService;
        $this->goalNotificationsService = $goalNotificationsService;
    }

    public function findNext(DTO\Request $request, DTO\Response $response){
        try{
            $params = $request->getParameters();

            Utilities::convertDynamicFilterToArray($params);
            if (array_key_exists("goalId", $params)) 
                $params["dynamic_filter"]["goalId"] = [["relation"=>  "AND", "operation" => "EQUALS", "value"=> $params["goalId"]]];

            $data = $this->goalNotificationsService->findNext($params, false);
            $data = Utilities::parseDataSourceZeedhiNext($data, $this->goalNotificationsService->countNext($params, false));

            // Paginação
            $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
            $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
            $this->sendResponse($data, 200);
        }catch (\Exception $e){
            throw  $e;
        }
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $request = $this->buildDataSetIfRow($request);
            $notification = $request->getDataSet()->getRows()[0];
            $notification = $this->goalNotificationsService->setNotificationNextDate($notification);
            $this->goalNotificationsService->saveNotification($notification);
            $response->addDataSet(new DataSet('notifications', array($notification)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 0));
        }
    }

    public function getGoalsInfo(DTO\Request\Row $request, DTO\Response $response){
        try {
            $dataSourceId = $request->getRow()['dataSourceId'];
            $notifications = $this->getGoalNotifications($dataSourceId, $response);
            $goals = $this->getGoalsByNotifications($notifications, $response);
            $this->getViewsByGoals($goals, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 0));
        }
    }

    protected function getGoalNotifications($dataSourceId, &$response) {
        $notifications = $this->goalNotificationsService->getGoalNotificationByDataSourceId($dataSourceId);
        $response->addDataSet(new DataSet('notifications', $notifications));
        return $notifications;
    }

    protected function getGoalsByNotifications($goalNotifications, &$response) {
        $goalIds = array();
        foreach ($goalNotifications as $notification) {
            array_push($goalIds, $notification['goalId']);
        }

        $goals =  $this->goalsService->getGoalsByIds($goalIds);
        $response->addDataSet(new DataSet('goals', $goals));
        return $goals;
    }

    protected function getViewsByGoals($goals, &$response) {
        $viewIds = array();
        foreach ($goals as $goal) {
            array_push($viewIds, $goal['viewId']);
        }

        $views = $this->viewService->getManyWidgetsById($viewIds);
        $response->addDataSet(new DataSet('views', $views));
    }
}
