import { LoadingService } from "@zeedhi/common";
import { Http } from '@zeedhi/core';

export class DataSourceService {
    public async saveDataSource(datasourceData: any, userId: any) {
        LoadingService.show();
        const res = await Http.post("/dataSource/save", {
            requestType: "DataSet",
            disableLoader: false,
            dataset: [datasourceData._id ? datasourceData : {
                ...datasourceData,
                __createdLocal: true,
                __is_new: true,
                createdBy: userId
            }]
        });
        LoadingService.hide();
        return res;
    }

    // Query Methods
    public extractStringsRangeFromQuery(code: string) {
        const stringRegex = /(["'])(?:(?=(\\?))\2.)*?\1/g;
        return [...code.matchAll(stringRegex)].map((codeString: any) => [codeString.index, codeString.index + codeString[0].length]);
    }

    public strReplaceNotIn(forbideenRange: any[], inForbideenFunc: any, notInForbideenFunc: any) {
        return (...params: any) => {
            const commentStart = params[params.length - 2];
            const containingString = forbideenRange.find(([start, end]: any) => commentStart > start && commentStart < end);
            if (containingString) {
                return inForbideenFunc(params, containingString);
            }
            return notInForbideenFunc(params);
        };
    }

    public clearComments(code: string): string {
        if (code === "") {
            return "";
        }
        const commentRegex = /(--.*)|(((\/\*)+?[\w\W]+?(\*\/)+))/g;
        const stringRanges = this.extractStringsRangeFromQuery(code);
        return code.replace(commentRegex, this.strReplaceNotIn(stringRanges, (params: any, [start, end]: any) => {
            const comment = params[0];
            const onString = comment.slice(0, end - start);
            const notOnString = comment.slice(end - start - 1, comment.length);
            return onString + this.clearComments(notOnString);
        }, () => ""));
    }

    public parseQueryCode(code: string): string {
        const doubleWhiteSpaceRegex = /\s+/g;
        const commentless = this.clearComments(code);
        const stringRanges = this.extractStringsRangeFromQuery(commentless);
        const parsedCode = commentless.replace(doubleWhiteSpaceRegex, this.strReplaceNotIn(stringRanges, (params: any) => params[0], () => " "));
        return parsedCode.trim();
    }
}
