import {Loader, SocketClient, SocketFactory} from '@zeedhi/core';
import {InnerSocket} from '@zeedhi/core/types/sockets/inner-socket';
import {DataSourceController} from "@/controllers/DataSource/DataSourceController";
import {DashboardController} from "@/controllers/DashboardController";

export class WebSocketClient extends SocketClient {
    private socketClient?: InnerSocket;

    private currentRoom?: string;

    private currentResolve: any;

    private currentReject: any;

    private room: string = 'deflaut';

    private wsNamespace: string = 'deflaut';

    private events: string[] = ['finishImportation', 'updateImportProcess', 'getCreatorDatasource'];

    public connectClient(url: string, room: string, wsNamespace: string) {
        this.room = room;
        this.wsNamespace = wsNamespace;
        const promise = new Promise((resolve) => {
            if (this.socketClient?.isOpen()) {
                this.socketClient?.close();
            }
            this.socketClient = SocketFactory.create(url);
            this.socketClient.addListener({
                onOpen: () => {
                    resolve('connected');
                },
                onMessage: (message: any) => {
                    if (message.event === 'finishImportation') {
                        this.finishImportation(message);
                    } else if (message.event === 'updateImportProcess') {
                        this.updateImportProcess(message);
                    }
                },
            });
        });

        return promise;
    }

    public close() {
        this.socketClient?.close();
    }

    public async connectClientZmart(url: string, room: string, wsNamespace: string) {
        await this.connectClient(url, room, wsNamespace);
        this.joinRoomZmart();
    }

    public finishImportation(data: any) {
        const DashboardCon = <DashboardController>Loader.getInstance('DashboardController');
        DashboardCon.finishImportation(data);
    }

    public updateImportProcess(data: any) {

        const DataSourceCon = <DataSourceController>Loader.getInstance('DataSourceController');
        DataSourceCon.updateImportProcess(data);
    }

    private getLogMessage(data: any) {
        let message = `'%cAction:%c ${data.action} | %cResponse:%c `;

        if (typeof data.data === 'object') {
            message += `["' ${data.data.join('", "')} '"] | `;
        } else {
            let value = data.data.replace(/\n/g, ' ');
            if (value !== '') {
                value = value.trim();
            }
            message += `"' ${value} '" | `;
        }

        message += data.isError ? '%cError' : '%cSuccess';

        return message;
    }

    public joinRoomZmart() {
        this.currentRoom = this.room;
        this.joinRoom(this.wsNamespace, this.room);
        this.addEventsZmart();
    }

    public joinRoom(namespace: string, room: string) {
        const eventsToStart = ['createRoom', 'joinRoom'];
        const socket = this.socketClient;
        const payload = {
            event: 'createNamespace',
            data: {
                key: 'you can create!',
            },
        };
        socket?.send(JSON.stringify({
            event: 'createNamespace',
            namespace,
            data: {
                key: 'you can create!',
            },
        }));
        eventsToStart.forEach((event: any) => {
            payload.event = event;
            this.send(payload);
        });
    }

    public addEventsZmart() {
        const eventsToStart = this.events;
        const payload = {
            event: 'on',
            data: {
                event: "finishImportation",
                key: 'you can create!',
            },
        };
        eventsToStart.forEach((event: any) => {
            payload.data.event = event;
            this.send(payload);
        });
    }

    private async send(payload: any) {
        const promise = new Promise((resolve, reject) => {
            this.currentResolve = resolve;
            this.currentReject = reject;
        });
        payload.room = payload.room || this.currentRoom;
        payload.namespace = payload.namespace || this.wsNamespace;
        this.socketClient?.send(JSON.stringify(payload));
        return promise;
    }
}
