import {Metadata, Http, Loader} from '@zeedhi/core';
import {
    Modal,
    ModalService,
    Form,
    Select,
    Grid,
    TextInput,
    AlertService,
    DialogService,
    CodeEditor,
    LoadingService,
    TreeGrid
} from '@zeedhi/common';
import Store from "@/store";

export class CollectorLogController {

    public Modal!: Modal;

    public async openModal() {
        if (this.Modal !== undefined) {
            await this.Modal?.destroy();
        }
        this.Modal = await ModalService.createFromJson('CollectorLog/modal/collectorLogModal', true);
        this.Modal.show();
    }

    public async closeModal() {
        this.Modal.hide();
        await this.Modal?.destroy();
    }

    public async updateLoggedUserInfo() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        const grid = Metadata.getInstance<TreeGrid>('log');
        await grid.reload();
    }

    public async showRowInfo({component} : any) {
        await this.openModal();
        const formModal = Metadata.getInstance<Form>("formCollectorLogModal");
        const formValue = component.datasource.currentRowValue;
        const request = { nameForm: formValue.name };
        formModal.value.nameCollectorLog = formValue.name;
        formModal.value.sizeCollectorLog = formValue.size;
        formModal.value.registerCollectorLog = "Loading...";
        const data = await Http.post('/collector-log-content', request);
        formModal.value.registerCollectorLog = data.data.data[0];
    }

    public logFieldOnClickButtonCopyLog({component} : any) {
        const form = this.Modal.getChildInstance<Form>("formCollectorLogModal");
        form.value.registerCopyCollectorLog = form.value.registerCollectorLog;
        const fieldCopy = form.getChildInstance<CodeEditor>("registerCopyCollectorLog");
        fieldCopy.staticCode = form.value.registerCopyCollectorLog;
        fieldCopy.copyToClipboard();
        AlertService.show({
            name: 'custom-alert',
            text: "Copiado com sucesso",
            right: true,
            top: true,
            color: "success"
        });

    }

}
