import {Metadata, Http, Loader} from '@zeedhi/core';
import {
    Modal,
    ModalService,
    Form,
    Select,
    Grid,
    TextInput,
    AlertService,
    DialogService,
    LoadingService,
    Loading,
    TreeGrid
} from '@zeedhi/common';
import { CustomAlertService } from '@/Services/CustomAlertService';

export class DefaultColumnsController {
    //
    private addModal !: Modal;
    private viewModal !: Modal;
    public viewInputDisable = true;
    private currentRow: any = {};
    //
    public reloadGrid() {
        const grid = Metadata.getInstance<Grid>('defaultcolumns-grid');
        grid.reload();
    }

    public async updateLoggedUserInfo() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        this.reloadGrid();
    }

    public async openAddModal() {
        if (this.addModal !== undefined) {
            await this.addModal?.destroy();
        }
        this.addModal = await ModalService.createFromJson('defaultColumns/addModal/addModal', true);
        this.addModal.show();
    }
    //
    public async closeAddModal() {
        this.addModal.hide();
        await this.addModal?.destroy();
    }
    //
    public async openViewModal() {
        if (this.viewModal !== undefined) {
            await this.viewModal?.destroy();
        }
        this.viewModal = await ModalService.createFromJson('defaultColumns/viewModal/viewModal', true);
        this.viewModal.show();
    }
    //
    public async closeViewModal() {
        this.viewModal.hide();
        await this.viewModal?.destroy();
    }
    //
    public async saveAddModal() {
        const form = Metadata.getInstance<Form>('add-form');
        if (!form.validate()) return;
        const requestBody = {
            disableLoader: false,
            requestType: "DataSet",
            dataset: [
                {
                    ...form.value,
                    changed: true,
                    _createdLocal: true,
                    _is_new: true
                }
            ]
        };
        LoadingService.show('Aguarde');
        await Http.post('defaultColumns/save', requestBody)
        .then(() => {
            this.reloadGrid();
            this.closeAddModal();
            CustomAlertService.saveSuccessAlert();
        }, (reason) => {
            CustomAlertService.errorAlert(reason);
        })
        .finally(() => {
            LoadingService.hide();
        });
    }
    //
    public changeInputViewModal() {
        const form = Metadata.getInstance<Form>('view-form');
        const dadosForm = form.children;
        dadosForm.forEach(function (forms: any) {
            forms.isVisible = false;
            if (form.value.type === 'dimension' || form.value.type === "" || form.value.type === null) {
                if (forms.dimension === true) {
                    forms.isVisible = true;
                }
            }

            if (form.value.type === 'fact') {
                if (forms.fact === true) {
                    forms.isVisible = true;
                }
            }
        });
    }

    public async showRowInfo({component, row} : any) {
        if (component.editing === true) return;
        if (row.type === null) {
            row.type = "";
        }
        await this.openViewModal();
        this.currentRow = row;
        Metadata.getInstance<Form>('view-form').value = row;
        this.viewInputDisable = false;
    }

    public getHexadecimalColor({row}: any) {

        return { backgroundColor: row.color, width: '100%' };
    }

    public async saveViewModal() {
        const formModal = Metadata.getInstance<Form>('view-form');
        const colorInput = [formModal.value.color];
        formModal.value.color = colorInput;
        const grid = Metadata.getInstance<Grid>('defaultcolumns-grid');
        const row = this.currentRow;
        if (!formModal.validate()) {
            return;
        }
        const requestBody = {
            disableLoader: false,
            requestType: "DataSet",
            dataset: [
                {
                    ...formModal.value,
                    changed: true,
                    _createdLocal: true,
                    _is_new: false,
                    _id: row._id
                }
            ]
        };
        LoadingService.show('Aguarde');
        await Http.post('defaultColumns/save', requestBody)
        .then(() => {
            this.reloadGrid();
            this.closeViewModal();
            CustomAlertService.saveSuccessAlert();
        }, (reason) => {
            CustomAlertService.errorAlert(reason);
        })
        .finally(() => {
            LoadingService.hide();
        });
    }
    //
    public showConfirmDelete({component, event}:any) {
        event.preventDefault();
        DialogService.show({
            name: 'confirm-delete',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o(s) registro(s) selecionado(s)?',
            buttons: [
                { name: 'nao', component: 'ZdButton', label: 'Não', events: { click: () => { DialogService.hide(); } }, outline: true },
                { name: 'sim', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteRow(); DialogService.hide(); } }},
            ],
        });
    }
    //
    private async deleteRow() {
        const grid = Metadata.getInstance<Grid>('defaultcolumns-grid');
        // for (let i = 0; i < grid.selectedRows.length; i += 1) {
        //     if (grid.selectedRows[i].base === true) {
        //         AlertService.show({
        //             name: "delete-alert",
        //             text: "Não é possível deletar estilo(s) padrão(ões)!",
        //             color: "error"
        //         });
        //         return;
        //     }
        // }
        const data = {
            "dataset": grid.selectedRows,
            "disableLoader": false,
            'requestType': 'DataSet'
        };
        LoadingService.show('Aguarde');
        await Http.post('defaultColumns/remove', data)
        .then(() => {
            CustomAlertService.deleteSuccessAlert();
            this.reloadGrid();
        }, (reason) => {
            CustomAlertService.errorAlert(reason);
        })
        .finally(() => {
            LoadingService.hide();
        });
    }
    //
    public async editInline({ component }: any) {
        const grid = component;
        const editedRows = component.getEditedRows().map((row: any) => ({ ...row, changed: true, _createdLocal: true, _is_new: false, _id: row._id }));
        if (editedRows?.length <= 0) {
            return;
        }
        const requestBody = {
            disableLoader: false,
            requestType: "DataSet",
            dataset: editedRows
        };
        LoadingService.show('Aguarde');
        const res = await Http.post('defaultColumns/save', requestBody)
        .then(() => {
            this.reloadGrid();
            CustomAlertService.saveSuccessAlert();
        })
        .catch((reason) => CustomAlertService.errorAlert(reason))
        .finally(() => LoadingService.hide());
    }
    //
    public adjustFilter({ component }: any) {
        const datasource = component.datasource;
        const filter = datasource.dynamicFilter;
        Object.entries(filter).forEach(([k, filters]: any) => {
            filter[k] = filters.filter(({value}: any) => !(value instanceof Array && value.length === 0));
            if (filter[k].length === 0) {
                delete filter[k];
            }
        });
    }
}
