import { Metadata, Http, DateHelper } from '@zeedhi/core';
import { SelectTree, Form } from '@zeedhi/common';
import { PeriodsService } from '@/Services/PeriodsService';

export class FilterModalController {
    private periodData: any = [];
    private periodElementId: number = -1;
    private readonly filterFormName: any = "filterForm";
    public periodsData: any = null;
    public periodHelperOptions: any = [];

    public onMountedFilterModal() {
        const filterStructure = localStorage.getItem('filterStructure');
        if (filterStructure) {
            const structureField = Metadata.getInstance<SelectTree>("structure_label");
            structureField.setValue(filterStructure);
            localStorage.setItem('filterStructure', "");
        }
    }

    public async adjustPeriodDropdown({ component }: any) {
        const periodDropdown = component.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;
    }

    protected async setPeriodsData() {
        if (!this.periodsData) {
            this.periodsData = await Http.post("/periods");
        }
        if (this.periodHelperOptions.length === 0) {
            const periodDate = this.periodsData;
            const data = periodDate.data.data;
            data.forEach((option: any) => DateHelper.register(option.periodsCode, option.name, () => {
                const service = new PeriodsService();
                const periods = service.decodePeriod(option.periodsCode);
                return [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
            }));
            this.periodHelperOptions = data.map((option: any) => option.periodsCode).filter((val: any) => val);
        }
    }

    public createPeriodElement(period: any = null, isLast: boolean = false, deleteButtonIsVisible: boolean = true) {
        this.periodElementId += 1;
        const periodFormat = period;
        let helperValue = "";
        if (typeof period === 'string') {
            helperValue = period;
        }
        let button: any = {
            "name": "addButton_0",
            "component": "ZdButton",
            "icon": true,
            "small": true,
            "iconName": "mdi-plus-circle-outline",
            "disabled": false,
            "isVisible": true,
            "events": { "click": this.addPeriod.bind(this) },
            "grid": { "cols": 1 },
        };
        if (deleteButtonIsVisible) {
            button = {
                "name": "deleteButton_" + this.periodElementId,
                "component": "ZdButton",
                "icon": true,
                "small": true,
                "iconName": "mdi-trash-can-outline",
                "disabled": false,
                "isVisible": true,
                "events": { "click": this.removePeriod.bind(this) },
                "grid": { "cols": 1 },
            };
        }
        return [{
            "name": "period_" + this.periodElementId,
            "component": "ZdDateRange",
            "label": "Período",
            "grid": { "cols": 11 },
            "value": periodFormat,
            "dateFormat": "DD/MM/YYYY",
            "inputFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "splitter": " - ",
            "orderedDates": true,
            "helperOptions": this.periodHelperOptions,
            "helperValue": helperValue,
            "events": { "onMounted": this.adjustPeriodDropdown.bind(this) }
        },
        {
            "name": "periodButtons_" + this.periodElementId,
            "component": "ZdContainer",
            "cssClass": "zd-pa-0 zd-ma-0 filterPeriodButton",
            "cssStyle": "margin: 0px !important; padding: 0px !important;",
            "children": [
                button
            ]
        }];
    }

    public addPeriod() {
        const form = Metadata.getInstance<Form>(this.filterFormName);
        const period: any = this.createPeriodElement();
        form.addChild(period[0]);
        form.addChild(period[1]);
    }

    public removePeriod({ component }: any) {
        const form = Metadata.getInstance<Form>(this.filterFormName);
        const id = component.name.replace("deleteButton_", "");
        form.removeChild("period_" + id);
        form.removeChild("periodButtons_" + id);
    }

    public async getChartViewFilterModalJson(filterModalParameters: any) {
        await this.setPeriodsData();
        localStorage.setItem('filterStructure', filterModalParameters.selectedStructure);
        let period = filterModalParameters.period ? filterModalParameters.period : null;
        if (period && !Array.isArray(period)) {
            period = [period];
        }

        const periodFields: any = [];
        if (period) {
            period.forEach((p: any, i: any) => {
                periodFields.push(...this.createPeriodElement(p, false, i > 0));
            });
        } else {
            periodFields.push(...this.createPeriodElement(null, false, false));
        }
        const filterFields: any = [
            {
               "name": "structure_label",
               "component": "ZdSelectTree",
               "label": "Estruturas Gerenciais",
               "dataValue": "_id",
               "dataText": "name",
               "parentField": "parent",
               "grid": {
                  "cols": 12
               },
               "value": filterModalParameters.selectedStructure,
               "datasource": {
                 "uniqueKey": "_id",
                 "data": filterModalParameters.structures,
             }
            },
            {
               "name": "dimensionFilter",
               "component": filterModalParameters.hasMultipleDimensions ? "ZdSelectMultiple" : "ZdSelect",
               "label": filterModalParameters.hasMultipleDimensions ? "Dimensões" : "Dimensão",
               "grid": {
                   "cols": 12
               },
               "dataText": "name",
               "dataValue": "id",
               "value": filterModalParameters.selectedDimension,
               "datasource": {
                 "data": filterModalParameters.dimensions
             }
            },
            {
               "name": "factsFilter",
               "component": filterModalParameters.hasMultipleFacts ? "ZdSelectMultiple" : "ZdSelect",
               "label": filterModalParameters.hasMultipleFacts ? "Fatos" : "Fato",
               "grid": {
                  "cols": 12
               },
               "returnObject": false,
               "dataText": "name",
               "dataValue": "id",
               "value": filterModalParameters.selectedFacts,
               "datasource": {
                 "data": filterModalParameters.facts
              }
            },
            {
               "name": "drillDownFilters",
               "component": "ZdSelectMultiple",
               "label": "DrillDown Filters",
               "grid": {
                  "cols": 12
               },
               "dataText": "name",
               "dataValue": "value",
               "returnObject": true,
               "value": filterModalParameters.drillDownFilters,
               "datasource": {
                 "data": filterModalParameters.drillDownFilters
             }
            },
            {
               "name": "filterByDimension",
               "component": "ZdSelectMultiple",
               "label": filterModalParameters.selectedDimension,
               "isVisible": false,
               "grid": {
                  "cols": 12
               },
               "datasource": {
                   "type": "simple",
                   "items": filterModalParameters.dimensionData ? filterModalParameters.dimensionData : null
               }
            },
        ];
        if (filterModalParameters.dimensions) {
            filterFields.push({
                "name": "basicUsageDivider",
                "component": "ZdDivider"
            });
            filterModalParameters.dimensions.forEach((dimension: any) => {
                if (filterModalParameters.dimensionFilterValues[dimension.id]) {
                    const selectedValue = filterModalParameters.drillDownFilters.find((filter: any) => filter.name === dimension.id);
                    filterFields.push({
                        "name": "filter_" + dimension.id,
                        "component": "ZdSelectMultiple",
                        "label": dimension.name,
                        "grid": {
                            "cols": 12
                        },
                        "value": selectedValue ? selectedValue.value : null,
                        "datasource": {
                            "type": "simple",
                            "items": filterModalParameters.dimensionFilterValues[dimension.id]
                        }
                    });
                }
            });
        }
        filterFields.push(
            {
                "name": "basicUsageDivider",
                "component": "ZdDivider"
             },
             ...periodFields
        );
        return {
            "name": "addStructureType",
            "persistent": true,
            "children": [
               {
                  "name": "backStructureTypeModal",
                  "component": "ZdHeader",
                  "color": "transparent",
                  "padless": true,
                  "elevation": 0,
                  "leftSlot": [
                     {
                        "name": "structureTypeName",
                        "component": "ZdText",
                         "tag": "h3",
                         "text": "Filtro"
                     }
                  ],
                  "rightSlot": [
                     {
                        "name": "closeFilterModal",
                        "component": "ZdButton",
                        "small": true,
                        "icon": true,
                        "flat": true,
                        "iconName": "close",
                        "events": {
                           "click": "{{ChartViewController.closeFilterModal}}"
                        }
                     }
                  ]
               },
               {
                  "name": this.filterFormName,
                  "component": "ZdForm",
                  "cssClass": "zd-my-2",
                  "cssStyle": "max-height: 450px !important",
                  "events": {
                    "onMounted": "{{FilterModalController.onMountedFilterModal}}"
                  },
                  "children": filterFields,
               },
               {
                  "name": "changeClienteModal",
                  "component": "ZdFooter",
                  "color": "transparent",
                  "padless": true,
                  "rightSlot": [
                     {
                        "name": "cancelFilterModal",
                        "component": "ZdButton",
                        "label": "Cancelar",
                        "outline": true,
                        "events": {
                           "click": "{{ChartViewController.closeFilterModal}}"
                        }
                     },
                     {
                        "name": "SAVE",
                        "component": "ZdButton",
                        "label": "OK",
                        "events": {
                         "click": "{{ChartViewController.applyFilter}}"
                      }
                     }
                  ]
               }
            ]
          };
    }
}
