import { Datasource, Metadata, Http, FormatterParserProvider, Loader, IEventParam, IDictionary } from '@zeedhi/core';
import { MasterDetail, Modal, ModalService, Grid, Form, CodeEditor, LoadingService, AlertService, DialogService, Tabs, SelectMultiple, Select, Container, Text, ITabsBeforeChangeEventParam } from '@zeedhi/common';
import { TekGrid, TekRestDatasource } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';

Metadata.get('metadata/ManagementStructures/StructureType/ManagementStructure/Modal/managementStructuresAddModal.json');
Metadata.get('metadata/ManagementStructures/Modal/managementStructuresAddResponsibleUserModal.json');

FormatterParserProvider.registerFormatter('column_usersResponsible', ({ row }: { row: any }) => {
    const names = Loader.getInstance('ManagementStructuresController');
    const namesVet = [];
    if (row.usersResponsible && row.usersResponsible !== '') {
      for (let i = 0; i <= row.usersResponsible.length; i += 1) {
          if (names.requestNames[row.usersResponsible[i]]) {
            namesVet.push(names.requestNames[row.usersResponsible[i]]);
          }
      }
    }
    return `${namesVet.join(', ')}`;
});

FormatterParserProvider.registerFormatter('column_accessProfiles', ({ row }: { row: any }) => {
    const accessProfiles = Loader.getInstance('ManagementStructuresController');
    const accessProfilesVet = [];
    if (row.accessProfiles && row.accessProfiles !== '') {
      for (let i = 0; i <= row.accessProfiles.length; i += 1) {
          if (accessProfiles.requestAccessProfiles[row.accessProfiles[i]]) {
            accessProfilesVet.push(accessProfiles.requestAccessProfiles[row.accessProfiles[i]]);
          }
      }
    }
    return `${accessProfilesVet.join(', ')}`;
});

export class ManagementStructuresController {

  private modal!: Modal;
  private addStructureTypeModal!: Modal;
  private addRowManagementStructureModal!: Modal;
  private importLoadSeveralStructuresFromQuery!: Modal;
  private managementStructuresAddModal!: Modal;
  private managementStructureEditModal!: Modal;
  private viewRowLogModal!: Modal;
  private logViewIconModal!: Modal;
  private importManagementStructureModal!: Modal;
  private addNewRowHierarchyTypeModal!: Modal;
  private rowClickHierarchyTypeHierarchyModal!: Modal;
  private relateFieldKeysModal!: Modal;
  private managementStructuresCurrentRow: any = {};
  public value: any = false;
  public disabled: any = true;
  public row: any = '';
  public nameManagementStructures: any = '';
  public usersNamesManagementStructures: any = [];
  public requestNames: any = '';
  public requestAccessProfiles: any = '';
  public name: string = '';
  public tekGridManagementStructures: any = '';
  public idTipoEstrutura: any = '';
  public showAnotherDetail = false;
  public responsibleUserValue: any = [];
  public structMessages: any = '';
  public StructureDados: any = '';
  public rowManagementStructureValue: any = '';
  public rowStructureType: any = '';
  public rowHierarchyTypeGrid: any = '';
  public managementStructureExternalId: any = '';
  public idHierarchyTypeGrid: any = '';
  public typeParentHierarchyType: any = '';
  public typeChildHierarchyType: any = '';
  public typeNameHierarchyType: any = '';
  public nameBackHierarchyType: any = '';
  public fieldKeyParentHierarchyType: any = '';
  public fieldKeyChildrenHierarchyType: any = '';

  get anotherMasterIsVisible() {
    return !this.showAnotherDetail;
  }

  private async deleteAndUpdate(datasource: Datasource, row: IDictionary<any>) {
    await datasource.delete(row);
    datasource.get();
  }

  public showAnotherMaster() {
    this.showAnotherDetail = false;
  }

  public async updateLoggedUserInfo() {
    const appController = Loader.getInstance('AppController');
    await appController.loginPromise.then();
    const grid = Metadata.getInstance<Grid>('managementStructuresTab1grid');
    grid.reload();
  }

  public rowClickHierarchyType({ event, component, row}: any) {

    const managementStructuresTab2grid = Metadata.getInstance<TekGrid>('managementStructuresTab2grid');
    const hierarchyTypeDetailTabs = Metadata.getInstance<Tabs>('hierarchyTypeDetailTabs');
    const hierarchyTypeTabForm = Metadata.getInstance<Form>('hierarchyTypeFormTab1');
    hierarchyTypeTabForm.children.forEach(function (formTabHierarchyType) {
      formTabHierarchyType.disabled = false;
    });
    Metadata.getInstance<Form>('hierarchyTypeFormTab1').value = component.datasource.currentRow;
    if (component.datasource.currentRow.isActive === 'Y') {
      Metadata.getInstance<Form>('hierarchyTypeFormTab1').value.isActive = true;
    }
    managementStructuresTab2grid.isVisible = false;
    hierarchyTypeDetailTabs.isVisible = true;
    event.preventDefault();
  }

  public async rowClick({ component, event, row }: any) {
    this.rowStructureType = '';
    this.rowStructureType = row;
    this.idTipoEstrutura = '';
    this.idTipoEstrutura = row._id;
    event.preventDefault();
    LoadingService.show();
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    managementStructures.activeTab = 0; // Setando a primeira Tab sempre como padrão - Selecionada.
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const headerCardFormStructuresManagement = Metadata.getInstance<Form>('headerCardFormStructuresManagement');
    const managementStructuresMasterDetailTab1FormResult = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');

    managementStructuresMasterDetailTab1FormResult.value.nameTextResult = row.name;
    managementStructuresMasterDetailTab1FormResult.value.activeCheckBox = row.isActive;
    managementStructuresMasterDetailTab1FormResult.value.query = row.query;

    await this.loadFieldKeysFromQueryEdit();

    const keyAreaTextResult = Metadata.getInstance<SelectMultiple>('keyAreaTextResult');
    const descriptionFieldtextResult = Metadata.getInstance<Select>('descriptionFieldtextResult');
    const conexionTextResult = Metadata.getInstance<Select>('connection');

    managementStructuresMasterDetailTab1FormResult.value.queryResult = row.query;
    await conexionTextResult.setValue(row.connection);
    await descriptionFieldtextResult.setValue(row.descriptionField);
    await keyAreaTextResult.setValue(row.fieldKey);

    if (row.isActive === "Y") {
      row.isActive = true;
    } else {
      row.isActive = false;
    }

    headerCardFormStructuresManagement.isVisible = true;
    managementStructures.isVisible = true; // Aparece o masterDetail
    managementStructuresTab1grid.isVisible = false; // some com grid
    managementStructuresMasterDetailTab1FormResult.isVisible = true; // Aparece form com o MasterDetail
    this.name = row.name;
    this.showAnotherDetail = true;
    LoadingService.hide();
  }

  public loadSeveralStructuresFromQueryCloseModal() {
    this.importLoadSeveralStructuresFromQuery?.destroy();
    this.importLoadSeveralStructuresFromQuery.hide();
  }

  public async loadSeveralStructuresFromQuery({component}: any) {
         let structMessage = "";
         this.structMessages = "";
         this.StructureDados = "";
         const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
         const data = managementStructuresTab1grid.datasource.data;
         LoadingService.show();
         const structure = await Http.post("/structure/getManyStructuresFromQuery", data);
         this.StructureDados = structure;
         LoadingService.hide();
         if (structure.data.length === 0) {
           DialogService.show({
                name: "base-edit-dialog",
                type: "info",
                title: "Atenção",
                text: "Não há estruturas para serem adicionadas.",
                cssStyle: "text-align: center"
            });
         } else {
            structure.data.forEach(function (str: any) {
              structMessage += str.typeName + " - " + str.name + "\n";
            });
            this.structMessages = structMessage;
            this.importLoadSeveralStructuresFromQuery = await ModalService.createFromJson('ManagementStructures/Modal/loadSeveralStructuresFromQuery', true);
            this.importLoadSeveralStructuresFromQuery.show();
         }
  }

  public async importManagementStructure() {
    this.importManagementStructureModal = await ModalService.createFromJson('ManagementStructures/StructureType/ManagementStructure/Modal/importManagementStructure', true);
    this.importManagementStructureModal.show();
    const formStructureType = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    const gridManagementStructures = Metadata.getInstance<TekGrid>('managementStructuresModalTab2TekGrid');

    const dados = {
      'filter': [
        {
          "name": "type",
          "operator": "=",
          "value": this.rowStructureType
        },
        {
          "name": "structures",
          "operator": "IN",
          "value": gridManagementStructures.datasource.data
        }
      ],
      'origin': {
          "containerLabel": "Estruturas Gerenciais",
          "widgetLabel": "Estruturas Gerenciais"
      },
      "requestType": "FilterData"
    };

    LoadingService.show();
    const data = await Http.post("/structure/getStructuresFromQuery", dados);
     setTimeout(() => {

      const gridImportManagementStructure = Metadata.getInstance<TekGrid>('importManagementStructureGrid');
      gridImportManagementStructure.datasource.data = data.data.dataset.structure;
      gridImportManagementStructure.datasource.uniqueKey = "externalIdString";
    }, 1000);
    LoadingService.hide();
  }

  public importManagementStructureClose() {
    this.importManagementStructureModal?.destroy();
    this.importManagementStructureModal.hide();
  }

  public loadSeveralStructuresFromQueryModal() {
    const loadSeveralStructuresFromQueryForm = Metadata.getInstance<Form>('loadSeveralStructuresFromQueryForm');
    loadSeveralStructuresFromQueryForm.value.textareaResult = this.structMessages;
  }

  public async saveImportManagementStructure() {
    const grid = Metadata.getInstance<TekGrid>('importManagementStructureGrid');
    const managementStructuresModalTab2TekGrid = Metadata.getInstance<TekGrid>('managementStructuresModalTab2TekGrid');

    const dados = {
      'dataset': grid.selectedRows,
      'orgin': {
          "containerLabel": "Estruturas Gerenciais",
          "widgetLabel": "Add Structures"
      },
      'requestType': "DataSet"
    };
    LoadingService.show();
    await Http.post("/structure/save", dados);
    LoadingService.hide();
    this.importManagementStructureClose();
    await managementStructuresModalTab2TekGrid.reload();
    CustomAlertService.saveSuccessAlert();
  }

  public async saveImportLoadSeveralStruct() {
    const dados = {
            'dataset': this.StructureDados.data,
            'requestType': 'DataSet'
        };
    LoadingService.show();
    await Http.post("/structure/save", dados);
    LoadingService.hide();
    this.loadSeveralStructuresFromQueryCloseModal();
    CustomAlertService.saveSuccessAlert();
  }

  public async loadFieldKeysFromQueryAdd() {
    const formModal = Metadata.getInstance<Form>('structureTypeForm');
    formModal.value.add = true;

    if (formModal.value.queryAdd) {
      LoadingService.show();
      try {
        const data = await Http.post("/structureType/getFieldKeysFromQuery", formModal.value);
        const fieldKey = Metadata.getInstance<SelectMultiple>('fieldKey');
        const descriptionField = Metadata.getInstance<Select>('descriptionField');
        fieldKey.datasource.updateData(data.data);// atribuindo valores para dentro do datasource(data)
        descriptionField.datasource.updateData(data.data);
        await Promise.all([fieldKey, descriptionField]);
      } catch (e) {
        CustomAlertService.errorAlert(e);
      }
      LoadingService.hide();
    }
  }

  public async loadFieldKeysFromQueryEdit() {
    const form = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    form.value.edit = true;

    if (form.value.query) {
      LoadingService.show();
      try {
        const data = await Http.post("/structureType/getFieldKeysFromQuery", form.value);
        const keyAreaTextResult = Metadata.getInstance<SelectMultiple>('keyAreaTextResult');
        const descriptionFieldtextResult = Metadata.getInstance<Select>('descriptionFieldtextResult');
        keyAreaTextResult.datasource.updateData(data.data);
        descriptionFieldtextResult.datasource.updateData(data.data);
        await Promise.all([keyAreaTextResult, descriptionFieldtextResult]);
      } catch (e) {
        CustomAlertService.errorAlert(e);
      }
      LoadingService.hide();
    }
  }

  public async getManagementStructuresValues(row: any) {
    this.row = row;
    this.nameManagementStructures = row.name;
    const managementStructures = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    const structureTypeUri = '/structureType';
    const structureTypeRequest = await Http.get(structureTypeUri);
    for (let i = 0; i <= structureTypeRequest.data.data.length; i += 1) {
      if (row.name === structureTypeRequest.data.data[i].name) {
         managementStructures.value.conexionTextResult = structureTypeRequest.data.data[i].connectionName;
         if (!structureTypeRequest.data.data[i].fieldKey) {
           structureTypeRequest.data.data[i].fieldKey = '';
         }
         managementStructures.value.keyAreaTextResult = structureTypeRequest.data.data[i].fieldKey;
         managementStructures.value.nameTextResult = structureTypeRequest.data.data[i].name;
         managementStructures.value.descriptionFieldtextResult = structureTypeRequest.data.data[i].descriptionField;
         if (structureTypeRequest.data.data[i].isActive === 'Y') {
            this.value = true;
         }
         managementStructures.value.queryResult = structureTypeRequest.data.data[i].query;
         managementStructures.value.conexionTextResult = structureTypeRequest.data.data[i]._id;
         break;
      }
    }
  }

  public editStructuresTypes() {
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const headerCardFormStructuresManagement = Metadata.getInstance<Tabs>('headerCardFormStructuresManagement');
    const children0 = managementStructures.tabs[0].children[0];
  }

  public async backToGridButtonHierarchyType() {
    const managementStructuresTab2grid = Metadata.getInstance<TekGrid>('managementStructuresTab2grid');
    const hierarchyTypeDetailTabs = Metadata.getInstance<Tabs>('hierarchyTypeDetailTabs');
    managementStructuresTab2grid.isVisible = true;
    hierarchyTypeDetailTabs.isVisible = false;
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    await managementStructuresTab1grid.reload();
  }

  public backToGridButton({ event, row }: any) {
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const headerCardFormStructuresManagement = Metadata.getInstance<Form>('headerCardFormStructuresManagement');
    const managementStructuresMasterDetailTab1FormResult = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    managementStructures.isVisible = false;
    managementStructuresTab1grid.isVisible = true;
    headerCardFormStructuresManagement.isVisible = false;// isVisible False, icone (seta) voltar.
  }

  public backButtonStructuresTypes(disabled: any) {
    const managementStructures = Metadata.getInstance<Tabs>('managementStructuresMasterDetailTabs');
    const children0 = managementStructures.tabs[0].children[0];
    if (children0.children) {
      for (let i = 0; i < children0.children.length; i += 1) {
        if (disabled === true) {
          children0.children[i].disabled = disabled;
          children0.children[6].isVisible = false;
        } else {
          children0.children[i].disabled = disabled;
          children0.children[6].isVisible = true;
        }
      }
    }
  }

  public async addNewRowHierarchyType() {
    this.addNewRowHierarchyTypeModal = await ModalService.createFromJson('ManagementStructures/HierarchyType/Modal/hierarchyTypeAddModal', true);
    this.addNewRowHierarchyTypeModal.show();
  }

  public hierarchyTypeAddCloseModal() {
    this.addNewRowHierarchyTypeModal?.destroy();
    this.addNewRowHierarchyTypeModal.hide();
  }

  public async addNewRowStructureTypeModal() {
    this.addStructureTypeModal = await ModalService.createFromJson('ManagementStructures/Modal/structureTypeAddModal', true);
    this.addStructureTypeModal.show();
  }

  public structureTypeCloseModal() {
    this.addStructureTypeModal?.destroy();
    this.addStructureTypeModal.hide();
  }

  public async addNewRowManagementStructureModal() {
    this.addRowManagementStructureModal = await ModalService.createFromJson('ManagementStructures/StructureType/ManagementStructure/Modal/managementStructuresAddModal', true);
    this.addRowManagementStructureModal.show();
  }

  public managementStructuresCloseModal() {
    this.addRowManagementStructureModal?.destroy();
    this.addRowManagementStructureModal.hide();
  }

  public async saveHierarchTypeModal() {
    const form = Metadata.getInstance<Form>('hierarchyTypeFormModal');
    const grid = Metadata.getInstance<Grid>('managementStructuresTab2grid');

    if (form.value.activeAddCheckBox === true) {
      form.value.activeCheckBox = "Y";
    }

    const dados = {
      'dataset': [
        {
          "changed": true,
          "isActive": form.value.activeAddCheckBox,
          "name": form.value.name,
          "parent": form.value.parentName,
          "typeChild": form.value.typeChildName,
          "typeParent": form.value.typeParentName,
          "__createdLocal": true,
          "__is_new": true
        }
      ],
      "requestType": "DataSet"
    };
    LoadingService.show();
    await Http.post("/hierarchyType/save", dados);
    await grid.datasource.get();
    await grid.reload();
    LoadingService.hide();
    this.messageSalveSuccess();
    this.hierarchyTypeAddCloseModal();
  }

  public async saveHierarchTypeEditModal({row}: any) {
    const form = Metadata.getInstance<Form>('editHierarchyTypeFormModal');
    const grid = Metadata.getInstance<Grid>('managementStructuresTab2grid');

    if (form.value.activeEditCheckBox === true) {
      form.value.activeEditCheckBox = "Y";
    }

    const dados = {
      'dataset': [
        {
          "changed": true,
          "isActive": form.value.activeEditCheckBox,
          "name": form.value.name,
          "parent": form.value.parentName,
          "structureTypeChild": this.rowHierarchyTypeGrid.structureTypeChild,
          "structureTypeParent": this.rowHierarchyTypeGrid.structureTypeParent,
          "typeChild": form.value.typeChildName,
          "typeParent": form.value.typeParentName,
          "_breadcrumb": this.rowHierarchyTypeGrid._breadcrumb,
          "_childCount": this.rowHierarchyTypeGrid._childCount,
          "_id": this.rowHierarchyTypeGrid._id,
          "_level": this.rowHierarchyTypeGrid._level
        }
      ],
      "requestType": "DataSet"
    };
    LoadingService.show();
    await Http.post("/hierarchyType/save", dados);
    await grid.datasource.get();
    await grid.reload();
    LoadingService.hide();
    this.messageSalveSuccess();
  }

  public async managementStructuresAdd() {
    const form = Metadata.getInstance<Form>('managementStructuresAddModalResult');
    const managementStructuresModalTab2TekGrid = Metadata.getInstance<TekGrid>('managementStructuresModalTab2TekGrid');
    const dados = {
            'dataset': [
                {
                  "name": form.value.nameContent,
                  "externalIdString": form.value.externalIdContent,
                  "typeName": form.value.typeContent,
                  "accessProfiles": form.value.accessProfilesAutoComplete,
                  "usersResponsible": form.value.responsibleUserAutoComplete,
                  "externalId": form.value.externalIdContent,
                  "type": this.idTipoEstrutura,
                  "Changed": true,
                  "generated": true
                }
              ],
      'requestType': 'DataSet'
    };
    LoadingService.show();
    // await Http.post("/structure/save", dados);
    await managementStructuresModalTab2TekGrid.datasource.get();
    await managementStructuresModalTab2TekGrid.reload();
    LoadingService.hide();
    this.messageSalveSuccess();
    this.managementStructuresCloseModal();
    await managementStructuresModalTab2TekGrid.datasource.get();
    await managementStructuresModalTab2TekGrid.reload();
  }

  public deleteRowManagementStructureModal() {
    event?.preventDefault();

    DialogService.show({
			name: 'deleteRowManagementStructureModal',
			type: 'warning',
			title: 'Confirmação da operação',
			text: 'Deseja excluir esta(s) linha(s)?',
			buttons: [
				{
					name: 'no',
					component: 'ZdButton',
					label: 'Não',
					outline: true,
					events: {
						click: () => {
							DialogService.hide();
						},
					},
				},
				{
					name: 'yes',
					component: 'ZdButton',
					label: 'Sim',
					events: {
						click: () => {
							this.deleteRowsManagementStructure();
							DialogService.hide();
						},
					},
				},
			],
		});
  }

  public async deleteRowsManagementStructure() {
     const rows = Metadata.getInstance<TekGrid>('managementStructuresModalTab2TekGrid');
     let request = {};
     const results = [];
     rows.selectedRows.forEach(function (row) {
        request = { dataset: [{_id: row._id, name: row.name, type: row.type, typeName: row.typeName, externalIdString: row.externalIdString, accessProfiles: row.accessProfiles, externalId: row.externalId, usersResponsible: row.usersResponsible}] };
        results.push(Http.post("/structure/remove", request));
     });
     await rows.datasource.get();
     await rows.reload();
  }

  public messageSalveSuccess() {
    AlertService.show({
            name: 'custom-alert',
            text: "Salvo com Sucesso",
            right: true,
            top: true,
            color: "success"
        });
  }

  public saveHierarchTypeTab() {
    const hierarchyTypeTabForm = Metadata.getInstance<Form>('hierarchyTypeFormTab1');
    hierarchyTypeTabForm.children.forEach(function (formTabHierarchyType) {
    formTabHierarchyType.disabled = true;
    });
    hierarchyTypeTabForm.children[5].isVisible = false;// Habilitando o botão salvar tab hierarchyTypeTabForm
  }

  public async saveAddHierarchType() {
    const form = Metadata.getInstance<Form>('structureTypeForm');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    let isActiveCheckBox = '';
    if (form.value.isActive === true) {
      isActiveCheckBox = 'Y';
    } else {
      isActiveCheckBox = 'N';
    }
     let data: any = {};
     data = {dataset: [{changed: false, descriptionField: form.value.descriptionField, fieldKey: form.value.fieldKey, isActive: isActiveCheckBox, connection: form.value.connectionName, loadFields: form.value.loadFields, name: form.value.name, query: form.value.queryAdd, __createdLocal: true, __is_new: true, disableLoader: false}], disableLoader: false, requestType: "DataSet"};
     LoadingService.show();
     await Http.post('/structureType/save', data);
     LoadingService.hide();
     this.messageSalveSuccess();
     this.structureTypeCloseModal();
     await managementStructuresTab1grid.reload();
  }

  public async saveButtonStructuresTypes() {
    await this.backButtonStructuresTypes(true);
    const managementStructures = Metadata.getInstance<Form>('managementStructuresMasterDetailTab1FormResult');
    const managementStructuresTab1grid = Metadata.getInstance<TekGrid>('managementStructuresTab1grid');
    const uriSaveStructures = '/structureType/save';
    let isActiveCheckBox = '';
    if (managementStructures.value.activeCheckBox === true) {
      isActiveCheckBox = 'Y';
    } else {
      isActiveCheckBox = 'N';
    }
    let data = {};

      data = {dataset: [{_id: this.idTipoEstrutura, changed: true, descriptionField: managementStructures.value.descriptionFieldtextResult, fieldKey: managementStructures.value.keyAreaTextResult, isActive: isActiveCheckBox, connection: managementStructures.value.connection, loadFields: managementStructures.value.descriptionFieldtextResult, name: managementStructures.value.nameTextResult, query: managementStructures.value.query, __createdLocal: true, __is_new: true, disableLoader: false}], disableLoader: false, requestType: "DataSet"};

    await Http.post(uriSaveStructures, data);
    this.messageSalveSuccess();
    LoadingService.show();
    await managementStructuresTab1grid.reload();
    LoadingService.hide();
    await this.backButtonStructuresTypes(false);
  }

    public setIndexTab({ component, tabIndex }: ITabsBeforeChangeEventParam) {
      const text = `${component.tabs[tabIndex].tabTitle} is selected`;
      AlertService.show({ name: 'alert', text: `${text}`, top: true, left: false, });
    }

    public clickTabHierarchyType({tabIndex}: any) {
      if (tabIndex === 1) {
        setTimeout(() => {
            const gridHierarchy = Metadata.getInstance<TekGrid>('hierarchyGrid');
            if (!(gridHierarchy.datasource instanceof TekRestDatasource)) return;

            gridHierarchy.datasource.dynamicFilter = {
              type: [
                {
                  operation: "EQUALS",
                  relation: "AND",
                  value: this.idHierarchyTypeGrid,
                }
              ]
            };
            gridHierarchy.datasource.get();
        }, 1000);
      }
    }

    public clickTab({tabIndex, element}: any) {

      if (tabIndex === 1) {

        setTimeout(() => {

          const managementStructuresModalTab2TekGrid = Metadata.getInstance<TekGrid>('managementStructuresModalTab2TekGrid');
          if (!(managementStructuresModalTab2TekGrid.datasource instanceof TekRestDatasource)) return;
          managementStructuresModalTab2TekGrid.datasource.dynamicFilter = {
              type: [
                {
                  operation: "EQUALS",
                  relation: "AND",
                  value: this.idTipoEstrutura,
                }
              ]
          };

          managementStructuresModalTab2TekGrid.datasource.get();
        }, 500);

      }

      if (tabIndex === 2) {

        setTimeout(() => {

          const structureTypeTabRegister = Metadata.getInstance<TekGrid>('structureTypeTabRegister');
          if (!(structureTypeTabRegister.datasource instanceof TekRestDatasource)) return;

          structureTypeTabRegister.datasource.dynamicFilter = {
              structureType: [
                {
                  operation: "EQUALS",
                  relation: "AND",
                  value: this.idTipoEstrutura,
                }
              ],
              collectionName: [
                {
                  operation: "IN",
                  relation: "AND",
                  value: ["structure"],
                }
              ]
          };

          structureTypeTabRegister.datasource.get();
        }, 500);
      }
   }

  public async findNames() {
     LoadingService.show();
     const uriStructure = '/structureNext';
     const uriZmartOperator = '/zmartOperator';
     const requestStructure = await Http.get(uriStructure);
     const dataValues = [];
     for (let i = 0; i < requestStructure.data.data.length; i += 1) {
        const request = {filter: [{operator: "IN", name: "CDOPERADOR", value: requestStructure.data.data[i].usersResponsible}], page: 1, requestType: "FilterData"};
        const data = Http.post(uriZmartOperator, request);
        dataValues.push(data);
     }
     const request = (await Promise.all(dataValues));
     const dataObject: any = {};
     for (let i = 0; i < request.length; i += 1) {
       if (request[i].data?.dataset?.zmartOperator && request[i].data.dataset.zmartOperator.length > 0) {
          for (let j = 0; j < request[i].data.dataset.zmartOperator.length; j += 1) {
            dataObject[request[i].data.dataset.zmartOperator[j].CDOPERADOR] = request[i].data.dataset.zmartOperator[j].NMOPERADOR;
          }
        }
      }
      this.requestNames = dataObject;
      this.findAccessProfiles();
      window.setTimeout(() => LoadingService.hide(), 2500);
  }

  public async findAccessProfiles() {
    const dataObject: any = {};
    const uriZmartAccessProfile = '/zmartAccessProfile';
    const request = await Http.get(uriZmartAccessProfile);
    for (let i = 0; i < request.data.data.length; i += 1) {
      dataObject[request.data.data[i].NRPERFILACESSO] = request.data.data[i].NMPERFILACESSO;
    }
    this.requestAccessProfiles = dataObject;
  }

  public async deleteRowHierarchyType({ component, event }: any) {
    event?.preventDefault();
    this.showMessageDeleteManagementStructure();
  }

  private back: any = {
    click: () => DialogService.hide(),
  };

  public showMessageDeleteManagementStructure() {
    DialogService.show({
      name: 'deleteRowManagementStructureModal',
      type: 'warning',
      title: 'Confirmação da operação',
      text: 'Deseja excluir esta(s) linha(s)?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.deleteManagementStructure();
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  public async deleteManagementStructure() {
    const row = Metadata.getInstance<TekGrid>('managementStructuresTab2grid');
    let request = {};
    const results = [];
    LoadingService.show();
     row.selectedRows.forEach(function (rows) {
       request = {dataset: [
          {
            _id: rows._id,
            name: rows.name,
            parent: rows.parent,
            parentName: rows.parentName,
            structureTypeChild: rows.structureTypeChild,
            structureTypeParent: rows.structureTypeParent,
            typeChild: rows.typeChild,
            typeChildName: rows.typeChildName,
            typeParent: rows.typeParent,
            typeParentName: rows.typeParentName,
            _childCount: rows._childCount,
            level: rows.level
          }],
          disableLoader: false,
          origin: {containerLabel: "Estruturas Gerenciais", containerName: "structure", widgetLabel: "Tipos de hierarquia", widgetName: "hierarchy_types"},
          requestType: "DataSet"};
       results.push(Http.post("/hierarchyType/remove", request));
     });

    await row.datasource.get();
    await row.reload();
    LoadingService.hide();
    CustomAlertService.deleteSuccessAlert();
  }

  public async editRowManagementStructureModal({row}: any) {
    this.managementStructureExternalId = '';
    this.managementStructureExternalId = row.externalId; // Pegando o external ID para deixar em formato de objeto na hora que for salvar.
    this.managementStructureEditModal = await ModalService.createFromJson('ManagementStructures/StructureType/ManagementStructure/Modal/managementStructuresEditModal', true);
    await this.managementStructureEditModal.show();
    this.rowManagementStructureValue = '';
    this.rowManagementStructureValue = row;
    LoadingService.show();
    const form = Metadata.getInstance<Form>('managementStructuresEdit');
    const usersResponsibleField = Metadata.getInstance<SelectMultiple>('responsibleUserAutoCompleteEdit');
    const accessProfilesField = Metadata.getInstance<SelectMultiple>('accessProfilesAutoCompleteEdit');
    form.value.nameEdit = row.name;
    form.value.externalIdEdit = row.externalIdString;
    await usersResponsibleField.setValue(row.usersResponsible);
    await accessProfilesField.setValue(row.accessProfiles);
    LoadingService.hide();
  }

  public closeEditRowManagementStructureModal() {
    this.managementStructureEditModal?.destroy();
    this.managementStructureEditModal.hide();
  }

  public async rowClickHierarchyTypeHierarchy({row}: any) {
    this.rowClickHierarchyTypeHierarchyModal = await ModalService.createFromJson('ManagementStructures/HierarchyType/Modal/rowClickHierarchyTypeHierarchy', true);
    await this.rowClickHierarchyTypeHierarchyModal.show();
    LoadingService.show();
    const form = Metadata.getInstance<Form>('formHierarchy');

    const structureParentName = Metadata.getInstance<Select>('structureParentName');
    if (!(structureParentName.datasource instanceof TekRestDatasource)) return;

    structureParentName.datasource.dynamicFilter = {
      type: [
        {
          operation: "EQUALS",
          relation: "AND",
          value: this.typeParentHierarchyType,
        }
      ]
    };

    const structureChildName = Metadata.getInstance<Select>('structureChildName');
    if (!(structureChildName.datasource instanceof TekRestDatasource)) return;

    structureChildName.datasource.dynamicFilter = {
      type: [
        {
          operation: "EQUALS",
          relation: "AND",
          value: this.typeChildHierarchyType,
        }
      ]
    };
    await structureParentName.datasource.get();
    await structureChildName.datasource.get();

    form.value.nameEditHierarchy = row.name;
    structureParentName.setValue(row.structureParent);
    structureChildName.setValue(row.structureChild);
    form.value.typeName = row.type;
    LoadingService.hide();
  }

  public beforeDeleteHierarchyTypeHierarchy({ event }: any) {
    event?.preventDefault();
    DialogService.show({
			name: 'beforeDeleteHierarchyTypeHierarchy',
			type: 'warning',
			title: 'Confirmação da operação',
			text: 'Deseja excluir esta(s) linha(s)?',
			buttons: [
				{
					name: 'no',
					component: 'ZdButton',
					label: 'Não',
					outline: true,
					events: {
						click: () => {
							DialogService.hide();
						},
					},
				},
				{
					name: 'yes',
					component: 'ZdButton',
					label: 'Sim',
					events: {
						click: () => {
							this.deleteHierarchyTypeHierarchy();
							DialogService.hide();
						},
					},
				},
			],
		});
  }
  public async deleteHierarchyTypeHierarchy() {
    const gridhierarchyGrid = Metadata.getInstance<TekGrid>('hierarchyGrid');
    const nameHierarchy = this.rowHierarchyTypeGrid.name;
    gridhierarchyGrid.selectedRows.forEach(function (row) {
      const dados = {
        'dataset': [
          {
            "name": row.name,
            "structureChild": row.structureChild,
            "structureChildName": row.structureChildName,
            "structureParent": row.structureParent,
            "structureParentName": row.structureParentName,
            "type": row.type,
            "typeName": nameHierarchy,
            "_id": row._id
          }
        ],
        'origin': {
          "containerName": "structure",
          "widgetName": "hierarchy",
          "widgetLabel": "Hierarquia",
          "containerLabel": "Estruturas Gerenciais"
        },
        'requestType': 'DataSet'
      };
      LoadingService.show();
      Http.post("/hierarchy/remove", dados);
      gridhierarchyGrid.reload();
    });
    await gridhierarchyGrid.reload();
    LoadingService.hide();
    CustomAlertService.deleteSuccessAlert();
  }

  public backGridHierarchyModal() {
    const tabsHierarchyTypeRowClick = Metadata.getInstance<Tabs>('tabsHierarchyTypeRowClick');
    const managementStructuresTab2Frame = Metadata.getInstance<Text>('managementStructuresTab2Frame');
    const editBackHierarchyTypeGrid = Metadata.getInstance<Text>('editBackHierarchyTypeGrid');
    managementStructuresTab2Frame.isVisible = true;
    tabsHierarchyTypeRowClick.isVisible = false;
    editBackHierarchyTypeGrid.isVisible = false;
  }

  public async editRowHierarchyType({row, event}: any) {
    if (event.target instanceof HTMLButtonElement) {
      return;
    }
    const tabsHierarchyTypeRowClick = Metadata.getInstance<Tabs>('tabsHierarchyTypeRowClick');
    const managementStructuresTab2Frame = Metadata.getInstance<Text>('managementStructuresTab2Frame');
    const editBackHierarchyTypeGrid = Metadata.getInstance<Text>('editBackHierarchyTypeGrid');
    tabsHierarchyTypeRowClick.activeTab = 0; // Setando a primeira Tab sempre como padrão - Selecionada.
    this.nameBackHierarchyType = "";
    this.nameBackHierarchyType = row.name;
    managementStructuresTab2Frame.isVisible = false;
    tabsHierarchyTypeRowClick.isVisible = true;
    editBackHierarchyTypeGrid.isVisible = true;
    this.typeParentHierarchyType = "";
    this.typeParentHierarchyType = row.typeParent;
    this.typeChildHierarchyType = "";
    this.typeChildHierarchyType = row.typeChild;
    this.typeNameHierarchyType = "";
    this.typeNameHierarchyType = row.name;
    this.idHierarchyTypeGrid = "";
    this.idHierarchyTypeGrid = row._id;
    this.rowHierarchyTypeGrid = "";
    this.rowHierarchyTypeGrid = row;
    const form = Metadata.getInstance<Form>('editHierarchyTypeFormModal');
    const typeParentName = Metadata.getInstance<Select>('typeParentName');
    const typeChildName = Metadata.getInstance<Select>('typeChildName');
    const parentName = Metadata.getInstance<Select>('parentName');

    form.value.name = row.name;
    if (row.isActive === "Y") {
      form.value.activeEditCheckBox = true;
    }
         LoadingService.show();
         await typeParentName.datasource.get();
         await typeChildName.datasource.get();
         await parentName.datasource.get();

         typeParentName.setValue(row.typeParent);
         typeChildName.setValue(row.typeChild);
         parentName.setValue(row.parent);
         LoadingService.hide();
  }

  public hierarchyTypeHierarchyEditCloseModal() {
    this.rowClickHierarchyTypeHierarchyModal?.destroy();
    this.rowClickHierarchyTypeHierarchyModal.hide();
  }

  public async saveEditRowManagementStructureModal() {
    const form = Metadata.getInstance<Form>('managementStructuresEdit');
    const grid = Metadata.getInstance<Grid>('managementStructuresModalTab2TekGrid');
    const dados = {
            'dataset': [
                {
                  "_id": this.rowManagementStructureValue._id,
                  "name": form.value.nameEdit,
                  "externalIdString": form.value.externalIdEdit,
                  "typeName": this.rowManagementStructureValue.typeName,
                  "accessProfiles": form.value.accessProfilesAutoCompleteEdit,
                  "usersResponsible": form.value.responsibleUserAutoCompleteEdit,
                  "externalId": this.managementStructureExternalId,
                  "type": this.rowManagementStructureValue.type, // External Id setado no método editRowManagementStructureModal. Deixando em formato objeto
                  "Changed": true,
                  "generated": true
                }
              ],
      'requestType': 'DataSet'
    };
    LoadingService.show();
    await Http.post("/structure/save", dados);
    LoadingService.hide();
    this.closeEditRowManagementStructureModal();
    await grid.datasource.get();
    await grid.reload();
    this.messageSalveSuccess();
  }

  public async viewRowLog({row}: any) {
    this.viewRowLogModal = await ModalService.createFromJson('ManagementStructures/StructureType/Log/Modal/viewRowLog', true);
    this.viewRowLogModal.show();
    setTimeout(() => {
          const form = Metadata.getInstance<Form>('viewRowLogForm');
          form.value = row;
          const valueAfter = JSON.stringify(row.VALUE_AFTER);
          form.value.VALUE_AFTER = valueAfter;
        }, 100);
  }

  public viewRowLogClose() {
    this.viewRowLogModal?.destroy();
    this.viewRowLogModal.hide();
  }

  public async logViewIcon({event, row}: any) {
    event.stopPropagation();
    this.logViewIconModal = await ModalService.createFromJson('ManagementStructures/StructureType/ManagementStructure/Modal/logViewIcon', true);
    this.logViewIconModal.show();

    setTimeout(() => {

      const logViewIconGrid = Metadata.getInstance<TekGrid>('logViewIconGrid');
      if (!(logViewIconGrid.datasource instanceof TekRestDatasource)) return;

      logViewIconGrid.datasource.dynamicFilter = {
        collectionId: [
          {
            operation: "EQUALS",
            relation: "AND",
            value: row._id,
          }
        ],
        collectionName: [
          {
            operation: "IN",
            relation: "AND",
            value: ["structure"],
          }
        ]
      };
    }, 100);
  }

  public logViewIconClose() {
    this.logViewIconModal?.destroy();
    this.logViewIconModal.hide();
  }

  public logViewIconDetail({row}: any) {
    const logViewIconGrid = Metadata.getInstance<TekGrid>('logViewIconGrid');
    const logViewIconDetail = Metadata.getInstance<Tabs>('logViewIconDetail');
    const form = Metadata.getInstance<Form>('logViewIconDetailForm');
    const backLogViewIcon = Metadata.getInstance<Form>('backLogViewIcon');
    backLogViewIcon.isVisible = true; // Colocando a seta como visivél, para poder ir nos detalhes.
    const valueAfter = JSON.stringify(row.VALUE_AFTER);
    const valueBefore = JSON.stringify(row.VALUE_BEFORE);
    form.value.treeValueDetail = row.TREE_VALUE;
    form.value.valueBeforeDetail = valueBefore;
    form.value.valueAfterDetail = valueAfter;
    form.value.cdOperadorDetail = row.CDOPERADOR;
    form.value.nmOperadorDetail = row.NMOPERADOR;
    form.value.nrorgDetail = row.NRORG;
    form.value.dsEmailOperDetail = row.DSEMAILOPER;
    form.value.dataUpdateDetail = row.DATA_UPDATE;
    logViewIconGrid.isVisible = false;
    logViewIconDetail.isVisible = true;
  }

  public backGridManagementStructure() {
    const logViewIconGrid = Metadata.getInstance<TekGrid>('logViewIconGrid');
    const logViewIconDetail = Metadata.getInstance<Tabs>('logViewIconDetail');
    const form = Metadata.getInstance<Form>('logViewIconDetailForm');
    const backLogViewIcon = Metadata.getInstance<Form>('backLogViewIcon');
    backLogViewIcon.isVisible = false; // Colocando a seta como invisivel, para poder voltar itens de detalhes.
    logViewIconGrid.isVisible = true;
    logViewIconDetail.isVisible = false;
  }

  public async relateFieldKeys() {
    this.relateFieldKeysModal = await ModalService.createFromJson('ManagementStructures/HierarchyType/Modal/relateFieldKeys', true);
    await this.relateFieldKeysModal.show();
  }

  public relateFieldKeysClose() {
    this.relateFieldKeysModal?.destroy();
    this.relateFieldKeysModal.hide();
  }

  public relateFieldKeysGenerateHierarchy() {
    const container = Metadata.getInstance<Container>('relateFieldKeysGenerateHierarchyList');
    const fieldKeyChildren = this.rowHierarchyTypeGrid.structureTypeChild.fieldKey;
    const fieldKeyParent = this.rowHierarchyTypeGrid.structureTypeParent.fieldKey;
    const arrayFieldKeysParent: any[] = [];
    this.fieldKeyParentHierarchyType = fieldKeyParent;
    this.fieldKeyChildrenHierarchyType = fieldKeyChildren;

    fieldKeyParent.forEach(function (keysParent: String) {
      arrayFieldKeysParent.push({
        "ID": keysParent,
        "NAME": keysParent
      });
    });

    let arrayFieldKeys: any[] = [];

    fieldKeyChildren.forEach(function (fieldKeysChildren: any) {
      arrayFieldKeys = ManagementStructuresController.relateFieldKeysGenerateHierarchyObject(fieldKeysChildren, fieldKeyParent, arrayFieldKeys, arrayFieldKeysParent);
    });
    container.children = arrayFieldKeys;
  }

  static relateFieldKeysGenerateHierarchyObject(fieldKeysChildren: String, fieldKeyParent: any, array: any[], arrayFieldKeysParent: any) {

    array.push({
                "name": fieldKeysChildren + "_nameText",
                "component": "ZdText",
                "text": fieldKeysChildren,
                "readOnly": true,
                "showLabel": false,
                "disabled": true,
                "cssStyle": "margin-bottom: 30px",
                "grid": {
                  "sm": 4
                }
              });

    array.push({
                "name": fieldKeysChildren + "_nameSelect",
                "component": "ZdSelect",
                "cssStyle": "margin-top: 6px",
                "showHelper": false,
                "showLabel": false,
                "dataText": "ID",
                "dataValue": "NAME",
                "grid": {
                  "sm": 8
                },
                "datasource": {
                  "data": arrayFieldKeysParent
                }
              });

    return array;
  }

  public async saveRelateFieldKeys() {
    const form = Metadata.getInstance<Form>('relateFieldKeysGenerateHierarchyList');
    const gridHierarchyTypeHierarchy = Metadata.getInstance<TekGrid>('hierarchyGrid');

    LoadingService.show();
    /* const dadosRemove = {
      'dataset': gridHierarchyTypeHierarchy.datasource.data,
      'origin': {
        "containerLabel": "Estruturas Gerenciais",
        "containerName": "structure",
        "widgetLabel": "Relate FieldKeys",
        "widgetName": "relations"
      },
      'requestType': "DataSet"
    }; */

    const dadosRemove = {
      'type': this.rowHierarchyTypeGrid._id,
      'origin': {
        "containerLabel": "Estruturas Gerenciais",
        "containerName": "structure",
        "widgetLabel": "Relate FieldKeys",
        "widgetName": "relations"
      },
      'requestType': "DataSet"
    };

    const formParentKey: any[] = [];
    this.fieldKeyChildrenHierarchyType.forEach(function (childrenKey: any) {
      if (form.value[childrenKey + "_nameSelect"] !== null) {
        formParentKey.push({
          "parentKey": form.value[childrenKey + "_nameSelect"],
          "childKey": childrenKey,
          "__isSelected": false
        });
      }
    });

    const row = {
      'row': {
          "fieldKeysRelation": formParentKey,
          "name": this.rowHierarchyTypeGrid.name,
          "parent": this.rowHierarchyTypeGrid.parent,
          "structureTypeChild": this.rowHierarchyTypeGrid.structureTypeChild,
          "structureTypeParent": this.rowHierarchyTypeGrid.structureTypeParent,
          "typeChild": this.rowHierarchyTypeGrid.typeChild,
          "typeChildName": this.rowHierarchyTypeGrid.typeChildName,
          "typeParent": this.rowHierarchyTypeGrid.typeParent,
          "typeParentName": this.rowHierarchyTypeGrid.typeParentName,
          "_id": this.rowHierarchyTypeGrid._id
      },
      'origin': {
        "containerLabel": "Estruturas Gerenciais",
        "widgetLabel": "Relate FieldKeys"
      },
      "requestType": "Row"
    };

    // await Http.post("/hierarchy/remove", dadosRemove);
    await Http.post("/hierarchy/removeAll", dadosRemove);
    gridHierarchyTypeHierarchy.reload();

    await Http.post("/hierarchy/generateHierarchies", row);
    gridHierarchyTypeHierarchy.reload();

    LoadingService.hide();
    this.relateFieldKeysClose();

  }
  /* public addNewManagementStructures() {
    this.openManagementStructuresAddModal();
  }

  public async createManagementStructuresAddResponsibleUserModal() {
    if (!this.managementStructuresAddResponsibleUserModal) {
       this.managementStructuresAddResponsibleUserModal = await ModalService.createFromJson('ManagementStructures/Modal/managementStructuresAddResponsibleUserModal', true);
    }
  }

  public openManagementStructuresAddResponsibleUserModal() {
    this.managementStructuresAddResponsibleUserModal.show();
  }

  public closeManagementStructuresAddResponsibleUserModal() {
    this.managementStructuresAddResponsibleUserModal.hide();
  }

  public async addUsersForUserResponsible() {
    const managementStructuresAddResponsibleUserModalTekGrid = Metadata.getInstance<TekGrid>('managementStructuresAddResponsibleUserModal-tek-grid');
    const select = Metadata.getInstance<SelectMultiple>('responsibleUserAutoComplete');
    const data = await select.datasource.updateData(managementStructuresAddResponsibleUserModalTekGrid.selectedRows);
    select.setValue(data);
    this.closeManagementStructuresAddResponsibleUserModal();
  }

  public async createManagementStructuresAddAccessProfilesModal() {
    if (!this.managementStructuresAddAccessProfilesModal) {
       this.managementStructuresAddAccessProfilesModal = await ModalService.createFromJson('ManagementStructures/Modal/managementStructuresAddAccessProfilesModal', true);
    }
  }

  public openManagementStructuresAddAccessProfilesModal() {
    this.managementStructuresAddAccessProfilesModal.show();
  }

  public closeManagementStructuresAddAccessProfilesModal() {
    this.managementStructuresAddAccessProfilesModal.hide();
  }

   public async addAccessProfilesForAccessProfiles() {
    const managementStructuresAddAccessProfiles = Metadata.getInstance<TekGrid>('managementStructuresAddAccessProfilesModal-tek-grid');
    const select = Metadata.getInstance<SelectMultiple>('accessProfilesAutoComplete');
    const data = await select.datasource.updateData(managementStructuresAddAccessProfiles.selectedRows);
    select.setValue(data);
    this.closeManagementStructuresAddAccessProfilesModal();
  } */

}
